
package use_trafficintersection;

import java.util.Timer;
import java.util.TimerTask;
import java.awt.*;
import java.awt.event.*; 
import javax.swing.*;
import use_trafficintersection.algorithm.Lights;
import use_trafficintersection.algorithm.Main;

/**
 * @author Sjoerd Westendorp
 */

/*
This class only communicates with a Renderer object
- This class is a JPanel, which can draw stuff on screen, since 
    RenderWorld is the only panel, it can draw on the entire screen.
- Also implements KeyListener, it can therefore receive key strokes
- Also implements ActionListener, allowing it to use timers.
- Defines the shape of the displayed roads.

How it works
- Registers the key listener
- Starts a timer that trigger 60 times per second
- Calls Renderer.loop() and Renderer.draw() every frame. 
    If renderer.pause is true, then it does not call loop().
- Pauses and unpauses the simulation.
- Draws the world and the topbar.
*/

public class RenderWorld extends JPanel implements KeyListener, MouseListener{ //ActionListener
    // Other objects
    public static Renderer renderer;
    public static Lights lights;
    
    // Settings
    public static final boolean SHOW_COLOR_PALLET = false;
    
    // Constants
    public static final int LANESIZE = 16; // old: 30
    public static final int SIDESIZE = 12; // Sidewalk size // old: 25
    public static final int LANECHANGESIZE = LANESIZE;
    public static final Font FONT = new Font("Helvetica", Font.PLAIN, 12);
    public static final int BUTTON_PAUSE = KeyEvent.VK_SPACE;
    
    // Timer length -> Defines frame rate
    public static final int TIMER_LENGTH = 1000/60;
    
    // Debug strings, set by the Renderer
    public String debugString0 = "";
    public String debugString1 = "";
    
    // Loop and draw timer
    private Timer timer;
    
    // Sliders
    public final Slider sliderCars;
    public final Slider sliderAutos;
    public final Slider sliderPeds;
    public final Slider sliderPacketLoss;
    public final Slider sliderSimSpeed;
    
    public final Slider sliderBiasU;
    public final Slider sliderBiasD;
    public final Slider sliderBiasL;
    public final Slider sliderBiasR;
    
    // Drawing Graphics
    Graphics gr;
    Graphics2D g2d;
    
    // ----------------------------------------------
    //		    CREATE RENDERER
    // ----------------------------------------------
    
    public RenderWorld(Renderer r,Lights l){
	renderer = r;
	lights = l;
	
	// Create sliders
	sliderCars = new Slider( Renderer.W / 2
		, 0
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Total cars"
		, 0f, 100f, 50f
	);
	
	sliderAutos = new Slider( Renderer.W * 3 / 4
		, 0
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Auto percentage"
		, 0f, 100f, 30f
	);
	
	sliderPeds = new Slider( Renderer.W / 2
		, Renderer.TOPBAR_ROW_SIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Total Pedestrians"
		, 0f, 100f, 50f
	);
	
	sliderPacketLoss = new Slider( Renderer.W * 3 / 4
		, Renderer.TOPBAR_ROW_SIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Packet Loss"
		, 0f, 100f, 10f
	);
	
	sliderSimSpeed = new Slider( Renderer.W / 2
		, Renderer.TOPBAR_ROW_SIZE*2
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Simulation Speed"
		, 1f, 30f, 1f
	);
	
	sliderBiasU = new Slider( Renderer.WORLD_SIZE/2 - Renderer.JUNCSIZE - Renderer.W / 4
		, Renderer.TOPBAR_SIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Traffic Bias (U)"
		, 0f, 100f, 100f
	);
	
	sliderBiasD = new Slider( Renderer.WORLD_SIZE/2 + Renderer.JUNCSIZE
		, Renderer.WORLD_SIZE + Renderer.TOPBAR_SIZE - Renderer.TOPBAR_ROW_SIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Traffic Bias (D)"
		, 0f, 100f, 100f
	);
	
	sliderBiasL = new Slider( 0
		, Renderer.WORLD_SIZE / 2 + Renderer.TOPBAR_SIZE + Renderer.JUNCSIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Traffic Bias (L)"
		, 0f, 100f, 100f
	);
	
	sliderBiasR = new Slider( Renderer.WORLD_SIZE - Renderer.W / 4
		, Renderer.WORLD_SIZE / 2 + Renderer.TOPBAR_SIZE - Renderer.JUNCSIZE - Renderer.TOPBAR_ROW_SIZE
		, Renderer.W / 4
		, Renderer.TOPBAR_ROW_SIZE
		, "Traffic Bias (R)"
		, 0f, 100f, 100f
	);
	
	// install sliders and key listener and obtain focus
        register();
        
	// Start timer
	timerStart();
    }
    
    private void register(){
	// install key listener and obtain focus
	addKeyListener(this);
	addMouseListener(this);
        setFocusable(true);
        requestFocus();
    }
    
    // ----------------------------------------------
    //		    DRAWING
    // ----------------------------------------------
    
    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g); 
	gr = g;
	g2d = (Graphics2D) g;
	g.setFont(FONT);
	
	// Time to draw
	drawWorld(g);
	renderer.draw();
	drawTopBar(g);
	drawAutoTesterOutput(g);
	if(renderer.debugData){
	    drawLaneCosts(g);
	}
	drawSystemStateCosts(g);
	
	// Draw debug
	g.setColor(Color.black);
        g.drawString(debugString0,5,Renderer.TOPBAR_SIZE+Renderer.TOPBAR_ROW_SIZE*2);
        g.drawString(debugString1,5,Renderer.TOPBAR_SIZE+Renderer.TOPBAR_ROW_SIZE*2+14);
      }
      
    private void drawWorld(Graphics g){
	// Grass
	g.setColor(new Color(0,180,0));
        g.fillRect(0, Renderer.TOPBAR_SIZE, Renderer.WORLD_SIZE, Renderer.WORLD_SIZE);
	
        // Sidewalks
        g.setColor(Color.GRAY);
	// L to R
        g.fillRect(0
		, Renderer.WORLD_SIZE/ 2 - 2*LANESIZE - SIDESIZE + Renderer.TOPBAR_SIZE
		, Renderer.WORLD_SIZE
		, 2*LANESIZE * 2 + SIDESIZE * 2
	);
	// U to D
	g.fillRect(Renderer.WORLD_SIZE/ 2 - 2*LANESIZE - SIDESIZE
		, Renderer.TOPBAR_SIZE
		, 2*LANESIZE * 2 + SIDESIZE * 2
		, Renderer.WORLD_SIZE
	);
        
	// Roads
	g.setColor(Color.BLACK);
	// L to R
        g.fillRect(0
		, Renderer.WORLD_SIZE/ 2 - 2*LANESIZE + Renderer.TOPBAR_SIZE
		, Renderer.WORLD_SIZE
		, 2*LANESIZE * 2
	);
	// U to D
	g.fillRect(Renderer.WORLD_SIZE/ 2 - 2*LANESIZE
		, Renderer.TOPBAR_SIZE
		, 2*LANESIZE * 2
		, Renderer.WORLD_SIZE
	);
	
	// Roads markers
	g.setColor(Color.YELLOW);
	// L
        g.fillRect(0
		, Renderer.WORLD_SIZE/ 2 - 1 + Renderer.TOPBAR_SIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 2
	);
	// R
        g.fillRect( Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, Renderer.WORLD_SIZE/ 2 - 1 + Renderer.TOPBAR_SIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 2
	);
	// U
	g.fillRect(Renderer.WORLD_SIZE/ 2 - 1
		, Renderer.TOPBAR_SIZE
		, 2
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);
	// D
	g.fillRect(Renderer.WORLD_SIZE/ 2 - 1
		, Renderer.TOPBAR_SIZE + Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, 2
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);
	
	// Second outbound lane remover
	Graphics2D glocal = (Graphics2D) g.create();
	glocal.setColor(Color.GRAY);
	int[] lanerem_x = {Renderer.convertToRenderX(Renderer.JUNCSIZE+LANECHANGESIZE)
		, Renderer.convertToRenderX(Renderer.WORLD_SIZE /2)
		, Renderer.convertToRenderX(Renderer.WORLD_SIZE /2)
		, Renderer.convertToRenderX(Renderer.JUNCSIZE+LANECHANGESIZE)
		, Renderer.convertToRenderX(LANESIZE * 2)
	};
	int[] lanerem_y = {Renderer.convertToRenderY(LANESIZE)
		, Renderer.convertToRenderY(LANESIZE)
		, Renderer.convertToRenderY(LANESIZE*2)
		, Renderer.convertToRenderY(LANESIZE*2)
		, Renderer.convertToRenderY(LANESIZE*2)
	};
	glocal.fillPolygon(lanerem_x, lanerem_y, lanerem_x.length);
	glocal.rotate(Math.PI / 2, Renderer.convertToRenderX(0), Renderer.convertToRenderY(0));
	glocal.fillPolygon(lanerem_x, lanerem_y, lanerem_x.length);
	glocal.rotate(Math.PI / 2, Renderer.convertToRenderX(0), Renderer.convertToRenderY(0));
	glocal.fillPolygon(lanerem_x, lanerem_y, lanerem_x.length);
	glocal.rotate(Math.PI / 2, Renderer.convertToRenderX(0), Renderer.convertToRenderY(0));
	glocal.fillPolygon(lanerem_x, lanerem_y, lanerem_x.length);
	
        // Remaining draw things
	drawLaneMarkers(g);
        drawTrafficLights(g);
        drawTrafficLightsW(g);
	if(SHOW_COLOR_PALLET)
	    colorPalletTest(g);
    }
    
    private void drawTopBar(Graphics g){
	// Background
	g.setColor(Color.WHITE);
	g.fillRect(0,0, Renderer.W, Renderer.TOPBAR_SIZE);
	
	// Bottum line
	g.setColor(Color.BLACK);
	g.drawLine(0,Renderer.TOPBAR_SIZE, Renderer.W, Renderer.TOPBAR_SIZE);
	
	// String 0
	String barString0 = "FPS: " + renderer.fps;
	barString0 += " | Draws: "+renderer.drawCount;
	barString0 += " | Loops: "+renderer.loopCount;
	barString0 += " | Pause: SPACE";
		
	// String 1
	String barString1 = "Cars: "+Renderer.mainobject.cars.size();
	barString1 += " | Pedestrians: "+Renderer.mainobject.peds.size();
	barString1 += " | Reset: R";
	
	// String 2
	String barString2 = "Car Time delay: "+Renderer.mainobject.timeSpent;
	barString2 += " (Avg. of last "+Renderer.mainobject.timeSpentList.size()+" cars)";
	
	// String 3
	String barString3 = "Total acceleration: "+Math.round(Renderer.mainobject.accel*100.0)/100.0;
	barString3 += " (Avg. of last "+Renderer.mainobject.accelList.size()+" cars)";
	
	// String 4
	String barString4 = "Finished cars counter: "+Renderer.mainobject.carCount;
	
	// Draw them
	g.setColor(Color.black); int y = 12, dy = 14;
        g.drawString(barString0,2,y); y+=dy;
        g.drawString(barString1,2,y); y+=dy;
	g.drawString(barString2,2,y); y+=dy;
	g.drawString(barString3,2,y); y+=dy;
	g.drawString(barString4,2,y); y+=dy;
	
	// Draw sliders
	sliderCars.draw(g);
	sliderAutos.draw(g);
	sliderPeds.draw(g);
	sliderPacketLoss.draw(g);
	sliderSimSpeed.draw(g);
	
	sliderBiasU.draw(g);
	sliderBiasD.draw(g);
	sliderBiasR.draw(g);
	sliderBiasL.draw(g);
    }
     
    private void drawAutoTesterOutput(Graphics g){
	int x = 20;
	int y = Renderer.WORLD_SIZE/2 + Renderer.TOPBAR_SIZE + 2*Renderer.JUNCSIZE;
	int maxdisplay = (Renderer.H - y - 10)/14;
	g.setColor(Color.black);
	
	if(AutoTester.output.size() <= 0){
	    g.drawString("Press A to start auto-testing",x,y);
	    return;
	}
	
	String s;
	int i_start = Math.max(0,AutoTester.output.size()-maxdisplay);
	for(int i = i_start; i < AutoTester.output.size(); i++){
	    s = AutoTester.output.get(i);
	    g.drawString(s,x,y);
	    y += 14;
	}
    }
    
    private void drawLaneCosts(Graphics g){
	int x = Renderer.WORLD_SIZE/2 + Renderer.JUNCSIZE + 10;
	int y = Renderer.WORLD_SIZE/2 + Renderer.TOPBAR_SIZE + Renderer.JUNCSIZE + 20;
	g.setColor(Color.black);
	
	g.drawString("U - "+lights.CostU+" - "+lights.getStateString(Lights.LIGHT_U)+" - t: "+lights.TimeU+" - et: "+lights.ETimeU, x, y); y+=14;
	g.drawString("L - "+lights.CostL+" - "+lights.getStateString(Lights.LIGHT_L)+" - t: "+lights.TimeL+" - et: "+lights.ETimeL, x, y); y+=14;
	g.drawString("D - "+lights.CostD+" - "+lights.getStateString(Lights.LIGHT_D)+" - t: "+lights.TimeD+" - et: "+lights.ETimeD, x, y); y+=14;
	g.drawString("R - "+lights.CostR+" - "+lights.getStateString(Lights.LIGHT_R)+" - t: "+lights.TimeR+" - et: "+lights.ETimeR, x, y); y+=14;
	
	g.drawString("WU - "+lights.CostWU+" - "+lights.getStateString(Lights.LIGHT_WU)+" - t: "+lights.TimeWU+" - et: "+lights.ETimeWU, x, y); y+=14;
	g.drawString("WL - "+lights.CostWL+" - "+lights.getStateString(Lights.LIGHT_WL)+" - t: "+lights.TimeWL+" - et: "+lights.ETimeWL, x, y); y+=14;
	g.drawString("WD - "+lights.CostWD+" - "+lights.getStateString(Lights.LIGHT_WD)+" - t: "+lights.TimeWD+" - et: "+lights.ETimeWD, x, y); y+=14;
	g.drawString("WR - "+lights.CostWR+" - "+lights.getStateString(Lights.LIGHT_WR)+" - t: "+lights.TimeWR+" - et: "+lights.ETimeWR, x, y); y+=14;
	
	g.drawString("RU - "+lights.CostRU+" - "+lights.getStateString(Lights.LIGHT_RU)+" - t: "+lights.TimeRU+" - et: "+lights.ETimeRU, x, y); y+=14;
	g.drawString("RL - "+lights.CostRL+" - "+lights.getStateString(Lights.LIGHT_RL)+" - t: "+lights.TimeRL+" - et: "+lights.ETimeRL, x, y); y+=14;
	g.drawString("RD - "+lights.CostRD+" - "+lights.getStateString(Lights.LIGHT_RD)+" - t: "+lights.TimeRD+" - et: "+lights.ETimeRD, x, y); y+=14;
	g.drawString("RR - "+lights.CostRR+" - "+lights.getStateString(Lights.LIGHT_RR)+" - t: "+lights.TimeRR+" - et: "+lights.ETimeRR, x, y); y+=14;
    }
    
    private void drawSystemStateCosts(Graphics g){
	int x = Renderer.WORLD_SIZE/2 + Renderer.JUNCSIZE + 10;
	int y = Renderer.TOPBAR_SIZE + 20;
	g.setColor(Color.black);
	
	float cost;
	int sx, tx;
	for(int i = 0; i < lights.systemStateCostList.size(); i++){
	    cost = lights.systemStateCostList.get(i);
	    
	    tx = x+25;
	    sx = x;
	    
	    if(lights.currentSystemState == i){
		g.drawString("S", sx, y);
		sx += 10;
	    }
	    if(lights.nextSystemState == i)
		g.drawString("N", sx, y);
	    
	    g.drawString(""+i, tx, y);
	    g.drawString("Cost: "+(float)(Math.round(cost*100.0f))/100.0f, tx+25, y); 
	    y+=14;
	}
	
	g.drawString(Main.autoTester.algo.toUpperCase(), x+25, y);
    }
    
    private void drawLaneMarkers(Graphics g){
	g.setColor(Color.DARK_GRAY);
	
	// L
        g.fillRect(0
		, Renderer.WORLD_SIZE/ 2 + Renderer.TOPBAR_SIZE + LANESIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 1
	);
	/*g.fillRect(0
		, Renderer.WORLD_SIZE/ 2 - 1 + Renderer.TOPBAR_SIZE - LANESIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 1
	);*/
	
	// R
        /*g.fillRect( Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, Renderer.WORLD_SIZE/ 2 + Renderer.TOPBAR_SIZE + LANESIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 1
	);*/
	g.fillRect( Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, Renderer.WORLD_SIZE/ 2 - 1 + Renderer.TOPBAR_SIZE - LANESIZE
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
		, 1
	);
	
	// U
	/*g.fillRect(Renderer.WORLD_SIZE/ 2 + LANESIZE
		, Renderer.TOPBAR_SIZE
		, 1
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);*/
	g.fillRect(Renderer.WORLD_SIZE/ 2 - 1 - LANESIZE
		, Renderer.TOPBAR_SIZE
		, 1
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);
	
	// D
	g.fillRect(Renderer.WORLD_SIZE/ 2 + LANESIZE
		, Renderer.TOPBAR_SIZE + Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, 1
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);
	/*g.fillRect(Renderer.WORLD_SIZE/ 2 - 1 - LANESIZE
		, Renderer.TOPBAR_SIZE + Renderer.WORLD_SIZE / 2 + 2*LANESIZE
		, 1
		, Renderer.WORLD_SIZE / 2 - 2*LANESIZE
	);*/
    }
    
    private void drawTrafficLights(Graphics g){
        // DOWN
        g.setColor(Lights.getColor(lights.LightD));
        g.fillRect(Renderer.convertToRenderX(2)
		, Renderer.convertToRenderY(2*LANESIZE)
		, LANESIZE - 3
		, Lights.DRAWSIZE
	);
	g.setColor(Lights.getColor(lights.LightRD));
	g.fillRect(Renderer.convertToRenderX(2+LANESIZE)
		, Renderer.convertToRenderY(2*LANESIZE)
		, LANESIZE - 3
		, Lights.DRAWSIZE
	);
        
        // UP
        g.setColor(Lights.getColor(lights.LightU));
        g.fillRect(Renderer.convertToRenderX(-LANESIZE+1)
		, Renderer.convertToRenderY(-2*LANESIZE-Lights.DRAWSIZE)
		, LANESIZE - 3
		, Lights.DRAWSIZE
	);
	g.setColor(Lights.getColor(lights.LightRU));
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE+1)
		, Renderer.convertToRenderY(-2*LANESIZE-Lights.DRAWSIZE)
		, LANESIZE - 3
		, Lights.DRAWSIZE
	);
        
        // LEFT
        g.setColor(Lights.getColor(lights.LightL));
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE-Lights.DRAWSIZE)
		, Renderer.convertToRenderY(2)
                , Lights.DRAWSIZE
		, LANESIZE - 3
	);
	g.setColor(Lights.getColor(lights.LightRL));
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE-Lights.DRAWSIZE)
		, Renderer.convertToRenderY(2+LANESIZE)
                , Lights.DRAWSIZE
		, LANESIZE - 3
	);
        
        // RIGHT
        g.setColor(Lights.getColor(lights.LightR));
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE)
		, Renderer.convertToRenderY(-LANESIZE+1)
                , Lights.DRAWSIZE
		, LANESIZE - 3
	);
	g.setColor(Lights.getColor(lights.LightRR));
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE)
		, Renderer.convertToRenderY(-2*LANESIZE+1)
                , Lights.DRAWSIZE
		, LANESIZE - 3
	);
	
    }
    
    private void drawTrafficLightsW(Graphics g){
        // DOWN
        g.setColor(Lights.getColor(lights.LightWD));
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE+1)
		, Renderer.convertToRenderY(2*LANESIZE+4)
		, Lights.DRAWWSIZE
		, SIDESIZE - 6
	);
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE-1-Lights.DRAWWSIZE)
		, Renderer.convertToRenderY(2*LANESIZE+4)
		, Lights.DRAWWSIZE
		, SIDESIZE - 6
	);
        
        // UP
        g.setColor(Lights.getColor(lights.LightWU));
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE+1)
		, Renderer.convertToRenderY(-2*LANESIZE+2-SIDESIZE)
		, Lights.DRAWWSIZE
		, SIDESIZE - 6
	);
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE-1-Lights.DRAWWSIZE)
		, Renderer.convertToRenderY(-2*LANESIZE+2-SIDESIZE)
		, Lights.DRAWWSIZE
		, SIDESIZE - 6
	);
        
        // LEFT
        g.setColor(Lights.getColor(lights.LightWL));
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE+2-SIDESIZE)
		, Renderer.convertToRenderY(2*LANESIZE+1)
		, SIDESIZE - 6
                , Lights.DRAWWSIZE
	);
        g.fillRect(Renderer.convertToRenderX(-2*LANESIZE+2-SIDESIZE)
		, Renderer.convertToRenderY(-2*LANESIZE-1-Lights.DRAWWSIZE)
		, SIDESIZE - 6
                , Lights.DRAWWSIZE
	);
        
        // RIGHT
        g.setColor(Lights.getColor(lights.LightWR));
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE+4)
		, Renderer.convertToRenderY(2*LANESIZE+1)
		, SIDESIZE - 6
                , Lights.DRAWWSIZE
	);
        g.fillRect(Renderer.convertToRenderX(2*LANESIZE+4)
		, Renderer.convertToRenderY(-2*LANESIZE-1-Lights.DRAWWSIZE)
		, SIDESIZE - 6
                , Lights.DRAWWSIZE
	);
    }
    
    private void colorPalletTest(Graphics g){
	g.setColor(Lights.getColor(Lights.RED));
	g.fillRect(10,100,10,10);
	g.setColor(Lights.getColor(Lights.GREEN));
	g.fillRect(10,110,10,10);
	g.setColor(Lights.getColor(Lights.YELLOW));
	g.fillRect(10,120,10,10);
	g.setColor(Color.RED);
	g.fillRect(20,100,10,10);
	g.setColor(Color.GREEN);
	g.fillRect(20,110,10,10);
	g.setColor(Color.YELLOW);
	g.fillRect(20,120,10,10);
	
    }
    
    // ----------------------------------------------
    //		    DRAWING BASE
    // ----------------------------------------------
    
    public void drawRoundRect(int x,int y,int w,int h,int r,Color c){
	gr.setColor(c);
	gr.fillRoundRect(x, y, w, h, r, r);
    }
    
    public void drawCross(int x,int y,int size,Color c){
	gr.setColor(c);
	gr.fillRect(x-size/2, y-1, size, 2);
	gr.fillRect(x-1, y-size/2, 2, size);
    }
    
    public void drawRect(int x,int y,int w, int h,Color c){
	gr.setColor(c);
	gr.fillRect(x-w/2, y-h/2, w, h);
    }
    
    public void drawRectRotate(int x,int y,int w, int h,Color c,double rotation){
	Graphics2D glocal = (Graphics2D) gr.create();
	
	glocal.setColor(c);
	glocal.rotate(rotation, x, y);
	glocal.fillRect(x-w/2, y-h/2, w, h);
    }
    
    public void drawCar(int x,int y,int w, int h,Color ccar,Color croof,double rotation){
	Graphics2D glocal = (Graphics2D) gr.create();
	
	glocal.rotate(rotation, x, y);
	
	// Draw wheels
	final int wheelsize = 2;
	glocal.setColor(Color.GRAY);
	glocal.fillRect(x-w/2,	    y-h/2+2,		2, wheelsize);
	glocal.fillRect(x-w/2,	    y+h/2-2-wheelsize,	2, wheelsize);
	glocal.fillRect(x+w/2-2,    y-h/2+2,		2, wheelsize);
	glocal.fillRect(x+w/2-2, y  +h/2-2-wheelsize,	2, wheelsize);
	
	// Draw Car
	glocal.setColor(ccar);
	glocal.fillRect(x-(w-2)/2, y-h/2, (w-2), h);
	
	// Draw roof
	final int roofmargin = 1;
	glocal.setColor(croof);
	glocal.fillRect(x-(w-2)/2+roofmargin, y-h/2+roofmargin, (w-2)-2*roofmargin, h-6*roofmargin);
    }
    
    public void drawPed(int x,int y,int r,Color c){
	// Lets keep it simple
	gr.setColor(c);
	gr.fillOval(x-r/2-1,y-r/2-1,r,r);
    }
    
    
    // ----------------------------------------------
    //		    TIMER HANDLER
    // ----------------------------------------------

    private void timerStart(){
	timer = new Timer();
	TimerTask task = new TimerTask(){
	    @Override
	    public void run() {
		SwingUtilities.invokeLater(new Runnable() {
		    @Override
		    public void run() { timerDone(); }
		});
	    }
	};  
	timer.scheduleAtFixedRate(task, TIMER_LENGTH, TIMER_LENGTH);
	
        //timer = new Timer(TIMER_LENGTH, this);
        //timer.start();
    }
    
    private void timerDone(){
	// Triggers when timer is done
	
	if( !renderer.paused ){
	    renderer.loop();
	}
	
	// Draw the screen again
	repaint();
    }
    
    //@Override
    //public void actionPerformed(ActionEvent e) {
    //}
    
    /*
    * @author Sjoerd Westendorp
    */
    
    // ----------------------------------------------
    //		    KEY LISTENER
    // ----------------------------------------------

    // Character buttons
    @Override
    public void keyTyped(KeyEvent e) {
	char key = e.getKeyChar();
	if( key == 'r')
	    renderer.reset();
	else if( key == 'd' )
	    renderer.debugData = !renderer.debugData;
	else if( key == 'a' )
	    AutoTester.useAutoTester = true;
    }

    // Button SPACE
    @Override
    public void keyPressed(KeyEvent e) {
	if( e.getKeyCode() == BUTTON_PAUSE){
	    togglePause();
	}
    }

    @Override
    public void keyReleased(KeyEvent e) {
    }
    
    private void togglePause(){
	renderer.paused = !renderer.paused;
	
	// Change window title
	if( renderer.paused )
	    renderer.setTitleAddition("- SIMULATION PAUSED");
	else
	    renderer.setTitleAddition("");
    }
    
    public void collsionPause(String text){
	renderer.paused = true;
	renderer.setTitleAddition("- !!! COLLISION !!! - " + text);
    }

    // ----------------------------------------------
    //		    MOUSE LISTENER
    // ----------------------------------------------
    
    @Override
    public void mouseClicked(MouseEvent e) {
    }

    // CONTAINS SLIDER CLICKS
    @Override
    public void mousePressed(MouseEvent e) {
	int x = e.getX();
	int y = e.getY();
	
	sliderCars.mouseClick(x,y);
	sliderAutos.mouseClick(x,y);
	sliderPeds.mouseClick(x,y);
	sliderPacketLoss.mouseClick(x,y);
	sliderSimSpeed.mouseClick(x,y);
	
	sliderBiasU.mouseClick(x,y);
	sliderBiasD.mouseClick(x,y);
	sliderBiasR.mouseClick(x,y);
	sliderBiasL.mouseClick(x,y);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }
}
